/*
   ***  Galaxy Mapper 2.0 xgalaxy.h
   ***  Compiled under ansi on SGI and Sun workstations
   ***
   ***  This is my first X program (and almost my first C) so don't send any
   ***  flames.  Any suggestions or improvements can be sent to me 
   ***  (since I'm not quite finished yet) at truscott@gfx.engga.uwo.ca
   ***
   ***  To compile type 'cc xgalaxy.c -o xgalaxy -lX11 -lm'
   ***
   ***  To run type 'xgalaxy <name of turn file> <name of home planet file>'
   ***  The default is 'xgalaxy turn homeworld'
   ***
   ***  What this program does it to show the galaxy map for the game
   ***  galaxy.  What it does for the game player is that it will show
   ***  who owns what planets (colour coded, New on version 1.0b),
   ***  and when you click you mouse by a planet it will tell
   ***  you the name of the planet and it's S points.  Also, if you click
   ***  on another planet, it will give you the S points again as well as
   ***  the distance (In light years) to the last planet that you clicked on.
   ***  Version 2.0 changes the parsing command to allow for future additions.
   ***  As well as fixing that colour problem.  This version also fixes the set
   ***  map size from only 200 to any size.
   ***
   ***  Enjoy.
   ***
   ***  Ashley A. Truscott

   ***  The file homeworld is in the format below

   	1    70.40   36.99
   	2   133.64    0.10
	3    96.77  142.23
	4    92.27  184.73
	5   130.93   34.76 
	etc...

*/


/* Declarations */

#ifndef SEEK_SET 
#define SEEK_SET 0
#endif

#ifndef SEEK_END 
#define SEEK_END 2
#endif

#define PLAYER struct player_stc
#define PLANET struct planet_stc

PLANET {
	char	name[80];
	float	x,y,s;
	};

PLAYER {
	PLANET 	**planet;
	char	playername[80], temp[80];
	int	numplanet;
	int	colour;
};

struct axises {
    	int 	x_axis;
    	int 	y_axis;
};

